import csv
import datetime
from django.http import HttpResponse
from django.shortcuts import render, get_object_or_404, redirect
from django.contrib import messages
from django.core.files.storage import FileSystemStorage
from django.core.paginator import Paginator
from django.views.decorators.http import require_POST, require_http_methods
from django.contrib.auth.decorators import login_required

from account.models import Account
from .models import ExpenseItem, ExpenseType
from django.db.models import Sum
from django.db.models.functions import TruncMonth


@login_required
@require_http_methods(["POST"])
def add_expense(request):
    try:
        name = request.POST.get('name')
        if not name:
            messages.error(request, 'Category name is required')
            return redirect('backoffice:expense-list')

        if ExpenseType.objects.filter(name=name).exists():
            messages.error(request, 'Category with this name already exists')
            return redirect('backoffice:expense-list')

        expense = ExpenseType.objects.create(name=name)
        messages.success(request, 'Category added successfully')
        return redirect('backoffice:expense-list')
    except Exception as e:
        messages.error(request, str(e))
        return redirect('backoffice:expense-list')


from django.shortcuts import render
from django.contrib import messages
from django.core.paginator import Paginator
import datetime
from .models import ExpenseItem, ExpenseType, Account

@login_required
def expense_list(request):
    # Get filter parameters from the GET request
    expense_type = request.GET.get('expense_type', '')
    transaction_id = request.GET.get('transaction_id', '')
    requested_by = request.GET.get('requested_by', '')
    start_date = request.GET.get('start_date')
    end_date = request.GET.get('end_date')
    accounts = Account.objects.all()

    # Base query for expenses
    expenses_query = ExpenseItem.objects.all().order_by('-created_at')

    # Apply filters to the query if parameters are provided
    if expense_type:
        expenses_query = expenses_query.filter(expense_type__name__icontains=expense_type)
    if transaction_id:
        expenses_query = expenses_query.filter(transaction_id=transaction_id)
    if requested_by:
        expenses_query = expenses_query.filter(requested_by__username__icontains=requested_by)  # Assuming Account has a 'username' field
    if start_date and end_date:
        start_date_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d").date()
        end_date_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d").date()
        expenses_query = expenses_query.filter(date__range=(start_date_obj, end_date_obj))

    # Pagination
    paginator = Paginator(expenses_query, 10)
    page_number = request.GET.get('page')
    expenses = paginator.get_page(page_number)

    # Get expense types for dropdown (optional)
    expense_types = ExpenseType.objects.all()

    context = {
        'expenses': expenses,
        "expense_types": expense_types,
        'expense_type': expense_type,
        'transaction_id': transaction_id,
        'requested_by': requested_by,
        'start_date': start_date,
        'end_date': end_date,
        'accounts': accounts
    }
    return render(request, 'backoffice/expenses/expense_management.html', context)

@login_required
@require_http_methods(["POST"])
def create_expense(request):
    try:
        expense_type = request.POST.get('expense_type')
        expense_amount = request.POST.get('amount')
        expense_date = request.POST.get('expense_date')
        description = request.POST.get('description')
        transaction_id = request.POST.get('transaction_id')
        requested_by_id = request.POST.get('requested_by')
        KRA = request.POST.get('KRA')
        requested_by = Account.objects.get(id=requested_by_id) if requested_by_id else None
        document = None
        if 'document' in request.FILES:
            uploaded_file = request.FILES['document']
            if uploaded_file.name.endswith(('.png', '.jpg', '.jpeg', '.pdf')):
                fs = FileSystemStorage()
                document = fs.save(uploaded_file.name, uploaded_file)
            else:
                messages.error(request, 'Invalid file format.')
                return redirect('backoffice:expense-list')

        expense_type_id = ExpenseType.objects.get(id=expense_type)

        if not expense_type or not expense_amount or not expense_date or not transaction_id:
            messages.error(request, 'Please fill in all required fields.')
            return redirect('backoffice:expense-list')

        expense = ExpenseItem(
            expense_type=expense_type_id,
            transaction_id=transaction_id,
            amount=expense_amount,
            date=expense_date,
            description=description,
            document=document,
            requested_by=requested_by,
            kra_etr = KRA
        )
        expense.save()
        messages.success(request, 'Expense Created Successfully!')
        return redirect('backoffice:expense-list')
    except Exception as e:
        messages.error(request, f"An error occurred: {str(e)}")
        return redirect('backoffice:expense-list')


@login_required
@require_http_methods(["POST"])
def update_expense(request, expense_id):
    try:
        expense = get_object_or_404(ExpenseItem, id=expense_id)
        expense_type = request.POST.get('expense_type')
        new_amount = request.POST.get('amount')
        new_date = request.POST.get('date')
        kra_etr = request.POST.get('kra_etr')
        new_description = request.POST.get('description')
        new_document = request.FILES.get('document')

        if expense_type and new_amount and new_date:
            expense.expense_type_id = expense_type
            expense.amount = new_amount
            expense.date = new_date
            expense.description = new_description
            expense.kra_etr = kra_etr

            if new_document:
                expense.document.delete(save=False)
                expense.document = new_document

            expense.save()
            messages.success(request, 'Expense updated successfully.')
        else:
            messages.error(request, 'Please fill in all required fields.')
        return redirect('backoffice:expense-list')
    except Exception as e:
        messages.error(request, f"An error occurred: {str(e)}")
        return redirect('backoffice:expense-list')


@login_required
@require_POST
def delete_expense(request, expense_id):
    try:
        expense = get_object_or_404(ExpenseItem, id=expense_id)
        expense.delete()
        messages.success(request, 'Expense deleted successfully')
    except Exception as e:
        messages.error(request, f"An error occurred: {str(e)}")
    return redirect('backoffice:expense-list')


@login_required
def export_expenses_csv(request):
    try:
        response = HttpResponse(content_type='text/csv')
        response['Content-Disposition'] = 'attachment; filename="expenses.csv"'

        writer = csv.writer(response)
        writer.writerow(['Expense Type', 'Amount', 'Date', 'Description', 'Created At', 'Updated At'])

        expenses = ExpenseItem.objects.all().values_list('expense_type__name', 'amount', 'date', 'description',
                                                         'created_at', 'updated_at')
        for expense in expenses:
            writer.writerow(expense)

        return response
    except Exception as e:
        messages.error(request, f"An error occurred: {str(e)}")
        return redirect('backoffice:expense-list')


@login_required
def dashboard_data(request):
    try:
        end_date = datetime.date.today()
        start_date = end_date - datetime.timedelta(days=30)

        total_expenses = ExpenseItem.objects.filter(date__range=[start_date, end_date]).aggregate(total=Sum('amount'))['total'] or 0

        expenses_category = ExpenseItem.objects.values('expense_type__name').annotate(total=Sum('amount'))

        top_categories = ExpenseItem.objects.values('expense_type__name').annotate(total=Sum('amount')).order_by('-total')[:5]

        year_ago = end_date - datetime.timedelta(days=365)
        trends = ExpenseItem.objects.filter(date__gte=year_ago) \
            .annotate(month=TruncMonth('date')) \
            .values('month') \
            .annotate(total=Sum('amount')) \
            .order_by('month')

        context = {
            'totalExpenses': total_expenses,
            'expensesByCategory': list(expenses_category),
            'topExpenseCategories': list(top_categories),
            'expenseTrends': list(trends)
        }
        return render(request, 'backoffice/expenses/dashboard.html', context)
    except Exception as e:
        messages.error(request, f"An error occurred: {str(e)}")
        return redirect('backoffice:expense-list')